#ifndef HYPER_GENOM_MODEL_GENOM_PROCESS_HH_
#define HYPER_GENOM_MODEL_GENOM_PROCESS_HH_

#include <genom_model/tclserv_client_queue.hh>

namespace hyper {
	namespace genom_model {

		struct genom_process_exception : public std::runtime_error
		{
			genom_process_exception(const std::string& name, 
									const std::string& why) :
				std::runtime_error("Can't start " + name + " : " + why)
			{}
		};

		class genom_process : public boost::noncopyable {
			std::string name;
			tclserv_client_queue& queue;
			int pid;

			public:
				genom_process(const std::string& name, tclserv_client_queue& queue) : 
					name(name), queue(queue) 
				{
					char* why = NULL;
					int err = tclserv_client_start_module_ex(
							queue.get_client_id(), name.c_str(), &why);
					if (err < 0) {
						std::string s(why);
						free(why);
						throw genom_process_exception(name, s);
					}
					pid = err;
				}

				bool is_alive() const {
					std::ostringstream oss;
					oss << "test -d /proc/" << pid;
					int err = tclserv_client_exec(queue.get_client_id(), oss.str().c_str(), NULL);
					return (err == 0);
				}

				~genom_process() 
				{
					tclserv_client_stop_module_ex(queue.get_client_id(), name.c_str(), NULL);
				}

		};
	}
}

#endif /* HYPER_GENOM_MODEL_GENOM_PROCESS_HH_ */
